;**************************************************************************
;*
;* FILE  format2.asm
;* Copyright (c) 1995, 1996, 2002, 2008, 2009, 2010
;*	Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: format2.asm,v 1.33 2010-03-31 19:58:55 tlr Exp $
;*
;* DESCRIPTION
;*   1540/41/70/71 Fast formatter MK II.
;*
;******
	processor 6502

	include	"macros.i"
	include	"format2_runtime.i"

	seg.u	zp
;**************************************************************************
;*
;* SECTION  zero page
;*
;******
	org	$f7
bank_zp:
	ds.b	1
ptr_zp:
	ds.w	1
num_zp:
	ds.b	1
shift_zp:
	ds.b	1
offs_zp:
	ds.b	1

	
	seg	code
;**************************************************************************
;*
;* Jump table
;*
;******
	org	$c000
_fmt2_format:
	jmp	.turbo_format
_fmt2_fetch_scan:
	jmp	.fetch_scan

; ident
	dc.b	"*** FORMAT II / TLR ***"

;**************************************************************************
;*
;* NAME  .turbo_format
;*
;* DESCRIPTION
;*   IN: $bb,$bc COMMAND  $b7=LEN   (as set by SETNAM)
;*   Carry=0 OK!  Carry=1 (couldn't format)
;*
;******
.turbo_format:
	cmp	#0
	bne	tf_skp1
	lda	#35
	ldx	#0
tf_skp1:
	jsr	set_params

	ldx	#16
	lda	#$a0
tf_lp0:	
	sta	fmt_nam_st-1,x
	dex
	bne	tf_lp0	
	
; null len?
	lda	$b7
	beq	tf_fl1

	ldy	#0
; check if format (N)
	lda	($bb),y
	cmp	#"N"
	bne	tf_fl1
	iny
	cpy	$b7
	beq	tf_fl1

; find and skip ':'
tf_lp1:
	lda	($bb),y
	iny
	cpy	$b7
	beq	tf_fl1
	cmp	#":"	
	bne	tf_lp1
	
; build name string
	ldx	#0
tf_lp2:
	lda	($bb),y
	cmp	#","
	beq	tf_skp2
	sta	fmt_nam_st,x
	inx
	iny
	cpy	$b7
	beq	tf_fl1
	cpx	#16	
	bne	tf_lp2
tf_skp2:
;* get id chars *
	ldx	#0
tf_lp3:
	iny
	cpy	$b7
	beq	tf_skp3
	lda	($bb),y
	sta	fmt_id_st,x
	inx
	cpx	#2	
	bne	tf_lp3

tf_skp3:
	jsr	init_drive

	clc
	rts
tf_fl1:
	sec
	rts


set_params:
	sta	fmt_conf_st+0	; tracks

	lda	#<$166
	cpx	#0
	beq	sp_skp1
	lda	#<$162	
sp_skp1:
	sta	fmt_conf_st+1	; capacity
	rts

;**************************************************************************
;*
;* NAME  fetch_scan
;*
;* DESCRIPTION
;*   Fetch scan data
;*
;******
.fetch_scan:
	stx	ptr_zp
	sty	ptr_zp+1
	lda	$ba		; drive #
	jsr	$ffb1		; listen
	lda	#$6f
	jsr	$ff93		; lstsec
	lda	#"M"
	jsr	$ffa8
	lda	#"-"
	jsr	$ffa8
	lda	#"R"
	jsr	$ffa8
	lda	#<fmt_scan
	jsr	$ffa8
	lda	#>fmt_scan
	jsr	$ffa8
	lda	#FMT2_SCAN_BUF_LEN
	jsr	$ffa8
	jsr	$ffae		; unlsn

	lda	$ba		; drive #
	jsr	$ffb4		; talk
	lda	#$6f
	jsr	$ff96		; tksa
	ldy	#0
fs_lp1:
	jsr	$ffa5
	sta	(ptr_zp),y
	iny
	cpy	#FMT2_SCAN_BUF_LEN
	bne	fs_lp1
	jsr	$ffab		; untalk
	rts
	
;**************************************************************************
;*
;* NAME  init_drive
;*
;* DESCRIPTION
;*   Install initial boot code in drive
;*
;******
MAX_M_W	equ	$20
NUM_M_W_CHUNKS	equ	[BOOT_CODE_LEN+MAX_M_W-1]/MAX_M_W
	INFO	"num M-W chunks",NUM_M_W_CHUNKS
init_drive:

	ldx	#NUM_M_W_CHUNKS
	ldy	#0
id_lp1:	
	lda	#"W"
	jsr	listen_send_m
	lda	tab_low-1,x
	jsr	$ffa8
	lda	tab_high-1,x
	jsr	$ffa8
	lda	#MAX_M_W
	jsr	$ffa8
	tya
	clc
	adc	#MAX_M_W
	sta	offs_zp
id_lp2:
	lda	boot_code_st,y
	jsr	$ffa8
	iny
	cpy	offs_zp
	bne	id_lp2
	
	jsr	$ffae		; unlsn

	dex
	bne	id_lp1
	
; run the code
	lda	#"E"
	jsr	listen_send_m
	lda	#<boot_code_entry
	jsr	$ffa8
	lda	#>boot_code_entry
	jsr	$ffa8
	jsr	$ffae		; unlsn

; synchronize the loader
 	sei
	lda	$dd00
	and	#%00000011
	sta	bank_zp
	tay
; wait for drive to signal busy so we know it's active
id_lp3:
	bit	$dd00		; DATA=1, CLK=0?
	bpl	id_lp3		; no, DATA=0, loop
	bvs	id_lp3		; no, CLK=1, loop
	tya
	ora	#%00100000	; DATA=0
	sta	$dd00
id_lp4:
	bit	$dd00		; CLK=1?
	bvc	id_lp4		; no, CLK=0, loop
	sty	$dd00		; DATA=1

; make sure we are re-entrant
	lda	#>[format_code_st-.OFFS]
	sta	id_sm1+2
	
.OFFS	equ	[-FORMAT_CODE_LEN]&$ff
.ENDPAGE equ	>[format_code_st+FORMAT_CODE_LEN]
	ldy	#.OFFS
id_lp5:
id_sm1:
	lda	format_code_st-.OFFS,y
	jsr	put_byte_boot
	iny
	bne	id_lp5
	inc	id_sm1+2
	lda	id_sm1+2
	cmp	#.ENDPAGE
	bne	id_lp5
	echo	"copy",format_code_st,format_code_st+FORMAT_CODE_LEN,.OFFS,.ENDPAGE

	cli
	
	rts


tab_low:
	dc.b	<[boot_code+2*MAX_M_W]
	dc.b	<[boot_code+1*MAX_M_W]
	dc.b	<[boot_code+0*MAX_M_W]
tab_high:
	dc.b	>[boot_code+2*MAX_M_W]
	dc.b	>[boot_code+1*MAX_M_W]
	dc.b	>[boot_code+0*MAX_M_W]

listen_send_m:
	pha
	lda	$ba		; drive #
	jsr	$ffb1		; listen
	lda	#$6f
	jsr	$ff93		; lstsec
	lda	#"M"
	jsr	$ffa8
	lda	#"-"
	jsr	$ffa8
	pla
	jmp	$ffa8


	subroutine	put_byte_boot
;**************************************************************************
;*
;* NAME  put_byte_boot
;*
;* DESCRIPTION
;*   Send byte to drive using slow 1-bit protocol
;*
;******
put_byte_boot:
	sta	shift_zp
.pb_lp1:
	bit	$dd00		; DATA=0?
	bpl	.pb_lp1		; yes, drive busy
	lda	bank_zp
	sta	.pb_sm2+1
	ora	#%00010000
	sta	.pb_sm1+1

	ldx	#4
.pb_lp2:
.pb_sm1:
	lda	#$00
	ror	shift_zp
	bcc	.pb_skp1
	ora	#%00100000
.pb_skp1:
	sta	$dd00		; CLK=0
	jsr	.pb_twelve
.pb_sm2:
	lda	#$00
	ror	shift_zp
	bcc	.pb_skp2
	ora	#%00100000
.pb_skp2:
	sta	$dd00		; CLK=1
	jsr	.pb_twelve
	dex
	bne	.pb_lp2
	lda	bank_zp
	sta	$dd00
.pb_twelve:
	rts

	
	subroutine	boot_code
;**************************************************************************
;*
;* NAME  boot_code
;*
;* DESCRIPTION
;*   Drive side boot code.  
;*   Receive real code using slow 1-bit protocol.
;*
;******
boot_code_st:
	rorg	$0146
.shift_zp	equ	$71
	
boot_code:

get_byte_boot:
	ldx	#4
	stx	$1800		; CLK=1, DATA=1
	START_SAMEPAGE	"get_byte_boot"
.gb_lp1:
	lda	#%00000100
.gb_lp2:
	bit	$1800		; CLK=0?
	beq	.gb_lp2		; no, wait
	lda	$1800
	lsr
	ror	.shift_zp
	lda	#%00000100
.gb_lp3:
	bit	$1800		; CLK=1?
	bne	.gb_lp3		; no, wait
	lda	$1800
	lsr
	ror	.shift_zp
	dex
	bne	.gb_lp1
	END_SAMEPAGE
	lda	#%00000010	; CLK=1, DATA=0
	sta	$1800
	lda	.shift_zp
	rts	
get_byte_boot_end:
	SHOWRANGE "get_byte_boot", get_byte_boot, get_byte_boot_end
		
boot_code_entry:
	sei
; signal that we have entered the boot code.
	lda	#%00001000
	sta	$1800		; CLK=0, DATA=1
; Spin up drive motor so it will be done by the time we have transferred
; the real code.
;	lda	#%00000100
	lsr
	ora	$1c00
	sta	$1c00		; Motor on

; Wait for the c64 to signal it is ready.
.bc_lp1:
	lda	$1800		; DATA=0?
	lsr
	bcc	.bc_lp1		; no, loop

.OFFS	equ	[-FORMAT_CODE_LEN]&$ff
.ENDPAGE equ	>[format_code+FORMAT_CODE_LEN]
	ldy	#.OFFS
.bc_lp2:
	jsr	get_byte_boot
.bc_sm1:
	sta	format_code-.OFFS,y
	iny
	bne	.bc_lp2
	inc	.bc_sm1+2
	lda	.bc_sm1+2
	cmp	#.ENDPAGE
	bne	.bc_lp2
	echo	"target",format_code,format_code+FORMAT_CODE_LEN,.OFFS,.ENDPAGE
boot_code_jmp		equ	.+1
boot_code_jmp_st	equ	boot_code_jmp+[boot_code_st-boot_code]
	jmp	format_code
boot_code_end:
	rend
BOOT_CODE_LEN	equ	.-boot_code_st
	SHOWRANGE "boot_code",boot_code,boot_code_end

;**************************************************************************
;*
;* SECTION  drive code
;*
;******

format_code_st:
	rorg	$0300
format_code:
	incbin	"fmt2_1541.prg",2
	rend
FORMAT_CODE_LEN	equ	.-format_code_st

fmt_scan	equ	$06e4
fmt_conf_st	equ	$06fc - format_code + format_code_st

fmt_bam_st	equ	$0700 - format_code + format_code_st
fmt_nam_st	equ	fmt_bam_st + $90
fmt_id_st	equ	fmt_bam_st + $a2
; eof
