;**************************************************************************
;*
;* FILE  format2_run.asm
;* Copyright (c) 2008, 2009, 2010 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: format2_run.asm,v 1.25 2010-03-31 20:05:11 tlr Exp $
;*
;* DESCRIPTION
;*   1540/41/70/71 Fast formatter MK II.
;*
;******
	processor 6502

	include	"macros.i"
	include	"format2_runtime.i"

	INFO	"--- start of file format2_run.asm ---"		

	seg.u	zp
;**************************************************************************
;*
;* SECTION  zero page
;*
;******
	org	$f7
y_zp:
	ds.b	1
cnt_zp:
	ds.b	1
tmp_zp:
	ds.b	1
gap_zp:
	ds.b	1
ptr_zp:
	ds.w	1
offs_zp:
	ds.b	1
opt_zp:
	ds.b	1

	seg	code
	org	$0801
;**************************************************************************
;*
;* Basic line!
;*
;******
StartOfFile:
	dc.w	EndLine
	dc.w	0
	dc.b	$9e,"2069 /T.L.R/",0
;	        0 SYS2069 /T.L.R/
EndLine:
	dc.w	0

	jmp	SysAddress

; date
	dc.b	$20,$10,$03,$31

;**************************************************************************
;*
;* SysAddress... When run we will enter here!
;*
;******
SysAddress:
; set device number to 8 if it is less than 8
	lda	#8
	cmp	$ba
	bcc	sa_skp1
	sta	$ba
sa_skp1:

	jsr	install_code

; fix screen
	jsr	$ff5b
	lda	#6
	sta	$d020
	lda	#0
	sta	$d021

sa_mainloop:
	lda	#<menu_msg
	ldy	#>menu_msg
	jsr	$ab1e

	lda	#<nameid_str
	ldy	#>nameid_str
	jsr	get_str_pre
	iny
	iny
	tya
	ldx	#<fmt_str
	ldy	#>fmt_str
	jsr	$ffbd

	lda	#<tailgap_msg
	ldy	#>tailgap_msg
	jsr	$ab1e
	lda	#<tailgap_opt
	ldy	#>tailgap_opt
	jsr	get_opt
	sta	tailgap

	lda	#<tracks_msg
	ldy	#>tracks_msg
	jsr	$ab1e
	lda	#<tracks_opt
	ldy	#>tracks_opt
	jsr	get_opt
	sta	tracks

; ask for disk to be inserted
	lda	#<insertdisk_msg
	ldy	#>insertdisk_msg
	jsr	$ab1e
sa_lp2:
	jsr	$ffe4
	cmp	#3
 	beq	sa_abort
	cmp	#13
	bne	sa_lp2
	beq	sa_format
sa_abort:
	lda	#<aborted_msg
	ldy	#>aborted_msg
	jsr	$ab1e
	jmp	sa_another

sa_format:
; perform formatting and extract status
	lda	#<formatting_msg
	ldy	#>formatting_msg
	jsr	$ab1e
	lda	tracks
	ldx	tailgap
	jsr	fmt2_format

	jsr	get_status

	jsr	get_scan_result

; check if we want to format again
sa_another:
	lda	#<another_msg
	ldy	#>another_msg
	jsr	$ab1e
	jsr	$ffcf
	cmp	#"Y"
	bne	sa_ex1
	jmp	sa_mainloop

sa_ex1:
; no, exit!
	jmp	($fffc)



;**************************************************************************
;*
;* NAME  messages
;*   
;******
menu_msg:
	dc.b	147
	dc.b	158		; yellow
	dc.b	"FORMAT II / TLR",13
	dc.b	129		; brown
	ds.b	15,$c3
	dc.b	13
	dc.b    158		; yellow
	dc.b	"20 SECONDS, FULL VERIFY, MEASURED GAPS",13
	dc.b	13
	dc.b	5		; white
	dc.b	"NAME,ID: ",0

	
tailgap_msg:
	dc.b	13,"TAILGAP: ",0
tailgap_opt:
	dc.b	2		; number of options
	dc.b	0		; currently selected option
	dc.w	tgo_str1
	dc.b	0
	dc.w	tgo_str2
	dc.b	1
tgo_str1:
	dc.b	"NORMAL",0
tgo_str2:
	dc.b	"OPTIMIZED",0

tracks_msg:
	dc.b	13,"TRACKS : ",0
tracks_opt:
	dc.b	3		; number of options
	dc.b	0		; currently selected option
	dc.w	tro_str1
	dc.b	35
	dc.w	tro_str2
	dc.b	40
	dc.w	tro_str3
	dc.b	41
tro_str1:
	dc.b	"35 (NORMAL)",0
tro_str2:
	dc.b	"40 (EXTENDED)",0
tro_str3:
	dc.b	"41 (EXTENDED)",0



insertdisk_msg:
	dc.b	13,13,"INSERT DISK AND PRESS ENTER...",13,0
	dc.b	0
aborted_msg:
	dc.b	"ABORTED!",13,0
formatting_msg:
	dc.b	13,"FORMATTING... ",13,0
another_msg:
	dc.b	13,5,"ANOTHER? Y",157,0



; formatting parameters			
fmt_str:
	dc.b	"N:"
nameid_str:
	ds.b	32,0
tailgap:
	dc.b	0
tracks:
	dc.b	0


;**************************************************************************
;*
;* NAME  get_str_pre
;*   
;* DESCRIPTION
;*   get string with pre recorded string.
;*   
;******
get_str_pre:
	sta	ptr_zp
	sty	ptr_zp+1
	ldx	$d3
	stx	tmp_zp
	jsr	$ab1e
	lda	tmp_zp
	sta	$d3
	ldy	#0
gst_lp1:
	jsr	$ffcf
	cmp	#13
	beq	gst_skp2
	sta	(ptr_zp),y
	iny
	bne	gst_lp1
gst_skp2:
	lda	#0
	sta	(ptr_zp),y
; Y=len
	rts
	

;**************************************************************************
;*
;* NAME  get_opt
;*   
;* DESCRIPTION
;*   get options from a list
;*   
;******
get_opt:
	sta	ptr_zp
	sty	ptr_zp+1
	ldy	#1
	lda	(ptr_zp),y
	sta	offs_zp

	ldy	$d3
	sty	y_zp
go_lp1:

; clear previous option
	ldy	y_zp
	lda	#" "
go_lp2:
	sta	($d1),y
	iny
	cpy	#40
	bne	go_lp2

; print option
	lda	#18
	jsr	$ffd2
	jsr	go_print
	lda	#146
	jsr	$ffd2

go_lp3:
	jsr	$ffe4
	cmp	#17
	beq	go_down
	cmp	#145
	beq	go_up
	cmp	#13
	bne	go_lp3	

; enter was pressed, fall through to update option.
go_print:
	ldy	y_zp
	sty	$d3

	lda	offs_zp
	asl
	adc	offs_zp
	adc	#2
; Acc=offs_zp*3 + 2
	pha
	tay
	lda	(ptr_zp),y
	pha
	iny
	lda	(ptr_zp),y
	tay
	pla
	jsr	$ab1e

	ldy	#1
	lda	offs_zp
	sta	(ptr_zp),y
	pla
	tay
	iny
	iny
	lda	(ptr_zp),y
	rts

go_down:
	inc	offs_zp
	lda	offs_zp
	ldy	#0
	cmp	(ptr_zp),y
	bne	go_lp1
	sty	offs_zp
	beq	go_lp1

go_up:
	dec	offs_zp
	bpl	go_lp1
	ldy	#0
	lda	(ptr_zp),y
	sta	offs_zp
	bne	go_up
	
;**************************************************************************
;*
;* NAME  get_status
;*   
;* DESCRIPTION
;*   get drive status and print to screen.
;*   
;******
get_status:
; initialize and open error channel
	lda	#15
	tay
	ldx	$ba
	jsr	$ffba	;SETLFS
	lda	#0
	jsr	$ffbd	;SETNAM
	jsr	$ffc0	;OPEN
	ldx	#15
	jsr	$ffc6	;CHKIN

; get error string from drive
gs_lp1:
	jsr	$ffcf	;CHRIN
	jsr	$ffd2
	cmp	#13
	bne	gs_lp1

; exit, clean up
	jsr	$ffcc	;CLRCH
	jmp	$ffe7	;CLALL
;	rts

;**************************************************************************
;*
;* NAME  get_scan_result
;*   
;* DESCRIPTION
;*   get scan result and print to screen.
;*   
;******
get_scan_result:
	ldx	#<scan_buf
	ldy	#>scan_buf
	jsr	fmt2_fetch_scan

	ldx	tracks
	stx	ttab2+3

	sec
	jsr	$fff0
	tya
	pha
	txa
	pha
	
	ldy	#0
	ldx	#15
	clc
	jsr	$fff0

	lda	#<head_msg
	ldy	#>head_msg
	jsr	$ab1e

	ldy	#0
	sty	cnt_zp
gsr_lp1:
	lda	#1
	jsr	print_pipe
	jsr	print_space

; print track range in zone
	ldy	cnt_zp
	ldx	ttab,y
	jsr	print_byte_adj
	jsr	print_dash
	ldx	ttab2,y
	jsr	print_byte

; if any problems in this zone, print a "*".
	lda	scan_buf+FMT2_LAST_STATUS
	cmp	#2
	bcc	gsr_skp1
	cpy	scan_buf+FMT2_LAST_ZONE
	bne	gsr_skp1
	lda	#"*"
	jsr	$ffd2
gsr_skp1:

; print table fields.
	jsr	print_estimated_capacity
	jsr	print_real_capacity
	jsr	print_intersector_gap
	jsr	print_real_tail

	lda	#38
	jsr	print_pipe
	
	jsr	print_cr

	inc	cnt_zp
	ldy	cnt_zp
	cpy	#4
	bne	gsr_lp1

	lda	#<tail_msg
	ldy	#>tail_msg
	jsr	$ab1e

	pla
	tax
	pla
	tay
	clc
	jsr	$fff0
	
	rts

head_msg:
	dc.b	13
	dc.b	158		; yellow
	dc.b	"            CAPACITY        GAPS",13
	dc.b	"   ZONE   EST.    REAL   SECT  TAIL",13
	dc.b	129		; brown
	dc.b	" "
	dc.b	176
	ds.b	36,195
	dc.b	174,13
	dc.b	0
	; 194
tail_msg:
	dc.b	129		; brown
	dc.b	" "
	dc.b	173
	ds.b	36,195
	dc.b	189,13
	dc.b	0

ttab:
	dc.b	 1,18,25,31
ttab2:
	dc.b	17,24,30,35

;**************************************************************************
;*
;* NAME  print_estimated_capacity
;*   
;* DESCRIPTION
;*   Print estimated capacity or "?" if not determined.
;*   
;******
print_estimated_capacity:
	ldy	cnt_zp
	lda	scan_buf+FMT2_SCAN_CAPL_TAB,y
	ora	scan_buf+FMT2_SCAN_CAPH_TAB,y
	beq	pec_fl1

	lda	#10
	sta	$d3
	ldy	cnt_zp
	ldx	scan_buf+FMT2_SCAN_CAPL_TAB,y
	lda	scan_buf+FMT2_SCAN_CAPH_TAB,y
	jmp	print_dec
;	rts
pec_fl1:
	lda	#12
	sta	$d3
	jmp	print_quo
;	rts

;**************************************************************************
;*
;* NAME  print_real_capacity
;*   
;* DESCRIPTION
;*   Print the range of real capacity or "?" if not determined.
;*   
;******
print_real_capacity:
	ldy	cnt_zp
	lda	scan_buf+FMT2_SCAN_GAPMAX_TAB,y
	cmp	scan_buf+FMT2_SCAN_GAPMIN_TAB,y
	bcc	prc_fl1		; gapmin > gapmax

	lda	#15
	sta	$d3
	lda	scan_buf+FMT2_SCAN_GAPMIN_TAB,y
	jsr	calc_track_len
	jsr	print_dec
	jsr	print_dash

	ldy	cnt_zp
	lda	scan_buf+FMT2_SCAN_GAPMAX_TAB,y
	jsr	calc_track_len
	jmp	print_dec
;	rts
prc_fl1:
	lda	#19
	sta	$d3
	jmp	print_quo
;	rts


;**************************************************************************
;*
;* NAME  print_intersector_gap
;*   
;* DESCRIPTION
;*   Print the calculated intersector gap or "?" if not determined.
;*   
;******
print_intersector_gap:
	ldy	cnt_zp
	lda	#26
	sta	$d3
	ldx	scan_buf+FMT2_SCAN_ISGAP_TAB,y
	beq	pig_fl1
	jmp	print_byte
;	rts
pig_fl1:
	jmp	print_quo
;	rts

;**************************************************************************
;*
;* NAME  print_real_tail
;*   
;* DESCRIPTION
;*   Print measured tail or "?" if not determined.
;*   
;******
print_real_tail:
	ldy	cnt_zp
	lda	scan_buf+FMT2_SCAN_GAPMAX_TAB,y
	cmp	scan_buf+FMT2_SCAN_GAPMIN_TAB,y
	bcc	prt_fl1		; gapmin > gapmax

	lda	#30
	sta	$d3
	ldx	scan_buf+FMT2_SCAN_GAPMIN_TAB,y
	jsr	print_byte
	jsr	print_dash
	ldx	scan_buf+FMT2_SCAN_GAPMAX_TAB,y
	jmp	print_byte
;	rts
prt_fl1:
	lda	#32
	sta	$d3
	jmp	print_quo
;	rts

;**************************************************************************
;*
;* NAME  calc_track_len
;*   
;* DESCRIPTION
;*   Calculate track length from tail gap + intersector gap.
;*   in: Acc=tail gap, cnt_zp = zone
;*   ret: X=low byte, Acc=high byte
;*
;******
sector_tab:
	dc.b	21,19,18,17
SECTOR_GCR_SIZE	equ	 $162

calc_track_len:
	ldx	cnt_zp
	ldy	scan_buf+FMT2_SCAN_ISGAP_TAB,x
	sty	gap_zp
	ldy	sector_tab,x
	sty	tmp_zp
	ldx	#0
ctl_lp1:
	clc
	adc	#<SECTOR_GCR_SIZE
	bcc	ctl_skp1
	inx
ctl_skp1:
	inx
	dey
	bne	ctl_lp1
; Acc/X = tail_gap + SECTOR_GCR_SIZE * num_sectors

	ldy	tmp_zp
	dey
ctl_lp2:
	clc
	adc	gap_zp
	bcc	ctl_skp2
	inx
ctl_skp2:
	dey
	bne	ctl_lp2
; Acc/X = tail_gap + SECTOR_GCR_SIZE * num_sectors + isgap * (num_sectors-1)
	
	stx	tmp_zp
	tax
	lda	tmp_zp
	rts

;**************************************************************************
;*
;* NAME  print_byte_adj, print_byte, print_dec
;*   
;* DESCRIPTION
;*   Print decimal numbers.
;*   
;******
print_byte_adj:
	cpx	#10
	bcs	pba_skp1
	jsr	print_space
pba_skp1:
print_byte:
	lda	#0
print_dec:
	sty	y_zp
	jsr	$bdcd
	ldy	y_zp
	rts

;**************************************************************************
;*
;* NAME  print_pipe, print_quo, print_dash, print_cr, print_space
;*   
;* DESCRIPTION
;*   Print various chars
;*   
;******
print_pipe:
	sta	$d3
	lda	#8
	sta	646
	lda	#194
	jsr	$ffd2
	lda	#7
	sta	646
	rts
print_quo:
	lda	#"?"
	dc.b	$2c
print_dash:
	lda	#"-"
	dc.b	$2c
print_cr:
	lda	#13
	dc.b	$2c
print_space:
	lda	#$20
	jmp	$ffd2


scan_buf:
	ds.b	FMT2_SCAN_BUF_LEN

;**************************************************************************
;*
;* SECTION  external code
;*   
;******
install_code:
	ldy	#>[FORMAT2_LEN+$ff]
	ldx	#0
ic_lp1:
ic_sm1:
	lda	format2_st,x
ic_sm2:
	sta	format2,x
	inx
	bne	ic_lp1
	inc	ic_sm1+2
	inc	ic_sm2+2
	dey
	bne	ic_lp1
	rts

format2_st:
	rorg	$c000
format2:
	incbin	"format2_c000.prg",2
FORMAT2_LEN	equ	.-format2
	rend
; eof
