;**************************************************************************
;*
;* FILE  format2_test.asm
;* Copyright (c) 2008 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: format2_test.asm,v 1.12 2010-03-30 09:16:11 tlr Exp $
;*
;* DESCRIPTION
;*   Test code for Format II.
;*
;******
	processor 6502

	include	"macros.i"
	include	"format2_runtime.i"

	INFO	"--- start of file format2_test.asm ---"		

	seg.u	zp
;**************************************************************************
;*
;* SECTION  zero page
;*
;******
	org	$20

	seg	code
	org	$0801
;**************************************************************************
;*
;* Basic line!
;*
;******
StartOfFile:
	dc.w	EndLine
	dc.w	0
	dc.b	$9e,"2069 /T.L.R/",0
;	        0 SYS2069 /T.L.R/
EndLine:
	dc.w	0
	
;**************************************************************************
;*
;* SysAddress... When run we will enter here!
;*
;******
SysAddress:
; set device number to 8 if it is less than 8
	lda	#8
	cmp	$ba
	bcc	sa_skp1
	sta	$ba
sa_skp1:

	jsr	install_code

; fix screen
	jsr	$ff5b
	lda	#14
	sta	$d020
	sta	646
	lda	#6
	sta	$d021
	jsr	prepare_system
	
	lda	#<menu_msg
	ldy	#>menu_msg
	jsr	$ab1e
sa_lp1:
	jsr	$ffe4
	cmp	#3
; 	beq	sa_ex1		
	cmp	#13
	bne	sa_lp1

; Turbo Format
	lda	#<formatting_msg
	ldy	#>formatting_msg
	jsr	$ab1e
	jsr	start_clock
	ldx	#<format_str
	ldy	#>format_str
	lda	#FORMATSTR_LEN
	jsr	$ffbd
	lda	#0
	jsr	fmt2_format
	php
	jsr	get_status
 	jsr	stop_clock	
	jsr	print_time
	lda	#<ok_msg
	ldy	#>ok_msg
	plp
	bcc	sa_skp2
	lda	#<formaterr_msg
	ldy	#>formaterr_msg
sa_skp2:
	jsr	$ab1e

	lda	#<menu2_msg
	ldy	#>menu2_msg
	jsr	$ab1e
sa_lp2:
	jsr	$ffe4
	cmp	#3
	beq	sa_ex1
	cmp	#13
	bne	sa_lp2

; Normal Format
	lda	#<formatting_msg
	ldy	#>formatting_msg
	jsr	$ab1e
	jsr	start_clock
	ldx	#<format_str
	ldy	#>format_str
	lda	#FORMATSTR_LEN
	jsr	$ffbd
	jsr	send_command
	php
	jsr	get_status
 	jsr	stop_clock	
	jsr	print_time
	lda	#<ok_msg
	ldy	#>ok_msg
	plp
	bcc	sa_skp3
	lda	#<formaterr_msg
	ldy	#>formaterr_msg
sa_skp3:
	jsr	$ab1e
	
	
			
	lda	#<done_msg
	ldy	#>done_msg
	jsr	$ab1e

	sei
	lda	$d020
sa_lp4:
	eor	#8
	sta	$d020
	jmp	sa_lp4
	
sa_ex1:
	jmp	$a474	

format_str:
	dc.b	"N0:TLR,01"
FORMATSTR_LEN	equ	.-format_str
	
menu_msg:
	dc.b	147,"FORMAT II TEST / TLR",13
	dc.b	"----------------------------------------"
menu2_msg:
	dc.b	13
	dc.b	"INSERT DISK AND PRESS ENTER...",13,0
	dc.b	0
formatting_msg:
	dc.b	13,"FORMATTING... ",0
ok_msg:
	dc.b	", OK",13,0
formaterr_msg:
	dc.b	", FORMAT ERR!",13,0
done_msg:
	dc.b	"DONE!",0


get_status:
; initialize and open error channel
	lda	#15
	tay
	ldx	$ba
	jsr	$ffba	;SETLFS
	lda	#0
	jsr	$ffbd	;SETNAM
	jsr	$ffc0	;OPEN
	ldx	#15
	jsr	$ffc6	;CHKIN

; get error string from drive
	ldy	#0
gs_lp1:
	jsr	$ffcf	;CHRIN
	jsr	$ffd2
	iny
	cmp	#13
	bne	gs_lp1

; exit, clean up
	jsr	$ffcc	;CLRCH
	jmp	$ffe7	;CLALL
;	rts


send_command:
; initialize, open error channel and send command
	lda	#15
	tay
	ldx	$ba
	jsr	$ffba	;SETLFS
	jsr	$ffc0	;OPEN

; exit, clean up
	lda	#15
	jsr	$ffc3	;CLOSE
	rts

;**************************************************************************
;*
;* NAME  print_hex
;*
;* DESCRIPTION
;*   output hex byte.
;*   
;******
print_hex:
	pha
	lsr
	lsr
	lsr
	lsr
	jsr	ph_skp1
	pla
	and	#$0f
ph_skp1:
	clc
	adc	#"0"
	cmp	#"9"+1
	bcc	ph_skp2
	adc	#"A"-("9"+1)-1
ph_skp2:
	jmp	$ffd2

;**************************************************************************
;*
;* NAME  start_clock, stop_clock
;*
;* DESCRIPTION
;*
;******
start_clock:
	sei
	lda	#0
	sta	$dd0e
	sta	$dd0f
	lda	#$7f
	sta	$dd0d
	lda	$dd0d

	ldx	#4
	lda	#$ff
stc_lp1:
	sta	$dd04-1,x
	dex
	bne	stc_lp1
	lda	#%10010001	;continuous count cpu cycles
	sta	$dd0e
	lda	#%01011001	;oneshot count timer A 0's
	sta	$dd0f

	cli
	rts

stop_clock:
	sei
	lda	#%00000000
	sta	$dd0e
	sta	$dd0f		

	ldx	#4
sc_lp1:
	lda	#$ff
	eor	$dd04-1,x
	sta	time-1,x	 
	dex
	bne	sc_lp1
	cli
	rts

time:
 	ds.b	4
	
;**************************************************************************
;*
;* NAME  print_time
;*
;* DESCRIPTION
;*
;******
TIMECONST_PAL	equ	985250*1000
timeconst_pal:
	dc.b	[TIMECONST_PAL>>0]&$ff,[TIMECONST_PAL>>8]&$ff
	dc.b	[TIMECONST_PAL>>16]&$ff,[TIMECONST_PAL>>24]&$ff
TIMECONST_NTSC	equ	1022730*1000
timeconst_ntsc:
	dc.b	[TIMECONST_NTSC>>0]&$ff,[TIMECONST_NTSC>>8]&$ff
	dc.b	[TIMECONST_NTSC>>16]&$ff,[TIMECONST_NTSC>>24]&$ff

pal_flag:
	dc.b	0
timeconst:
	ds.b	4
count:
	ds.b	1
lead:
	ds.b	1
print_time:
; calculate
	lda	#0
	sta	count
	sta	lead
	ldy	#4
	lda	pal_flag
	bne	pt_skp1
	ldy	#4+[timeconst_ntsc-timeconst_pal]
pt_skp1:
	ldx	#4
pt_lp1:
	lda	timeconst_pal-1,y
	sta	timeconst-1,x
	dey
	dex
	bne	pt_lp1
pt_lp2:
	
	ldx	#4
pt_lp3:
	lda	time-1,x
	sta	dividend-1,x
	lda	timeconst-1,x
	sta	divisor-1,x
	dex
	bne	pt_lp3
	jsr	divide

	ldx	#4
	lda	#0
pt_lp4:
	sta	divisor-1,x
	dex
	bne	pt_lp4
	lda	#10
	sta	divisor
	jsr	divide

	lda	count
	cmp	#4
	bne	pt_skp3
	lda	lead
	bne	pt_skp2
	lda	#"0"
	jsr	$ffd2
pt_skp2:
	lda	#"."
	jsr	$ffd2
	sta	lead
pt_skp3:
	lda	lead
	ora	remainder
	beq	pt_skp4
	sta	lead
	lda	remainder
	ora	#$30
	jsr	$ffd2
pt_skp4:

	ldx	#4
pt_lp5:
	lda	timeconst-1,x
	sta	dividend-1,x
	dex
	bne	pt_lp5
	jsr	divide

	ldx	#4
pt_lp6:
	lda	dividend-1,x
	sta	timeconst-1,x
	dex
	bne	pt_lp6

	inc	count
	lda	count
	cmp	#6
	bne	pt_lp2	

	rts

;**************************************************************************
;*
;* NAME  divide
;*
;* DESCRIPTION
;*   32 bit divide
;*   dividend[0-3]=dividend[0-3]/divisor[0-3]
;*   remainder[0-3]=dividend[0-3]%divisor[0-3]
;*   carry set = divide by zero!
;*
;******
divisor:
	ds.b	4
dividend:
	ds.b	4
remainder:
	ds.b	4
divide:
;*
;* Check for zero dividend
;*
	lda	divisor
	ora	divisor+1
	ora	divisor+2
	ora	divisor+3
	sec
	beq	div_fl1		;exit with carry=1 if zero

;*
;* setup remainder
;*
	lda	#0
	sta	remainder
	sta	remainder+1
	sta	remainder+2
	sta	remainder+3

	ldx	#32+1
	clc
	bcc	div_skp1	;ALWAYS!

;*
;* divide loop
;*
div_lp1:
	rol	remainder
	rol	remainder+1
	rol	remainder+2
	rol	remainder+3

;*
;* Does it fit?
;*
	sec
	lda	remainder
	sbc	divisor
	lda	remainder+1
	sbc	divisor+1
	lda	remainder+2
	sbc	divisor+2
	lda	remainder+3
	sbc	divisor+3
	bcc	div_skp1	;no... shift in 

;* yes... subtract
	sec
	lda	remainder
	sbc	divisor
	sta	remainder
	lda	remainder+1
	sbc	divisor+1
	sta	remainder+1
	lda	remainder+2
	sbc	divisor+2
	sta	remainder+2
	lda	remainder+3
	sbc	divisor+3
	sta	remainder+3
div_skp1:
	rol	dividend
	rol	dividend+1
	rol	dividend+2
	rol	dividend+3
	dex
	bne	div_lp1
	clc
div_fl1:
	rts

;**************************************************************************
;*
;* NAME  prepare_system
;*
;* DESCRIPTION
;*   Prepare
;*
;******
prepare_system:	
	php
	sei

	jsr	check_pal
	lda	#0
	rol
	sta	pal_flag
	plp
	rts

;**************************************************************************
;*
;* NAME  wait_vb
;*
;* DESCRIPTION
;*   Restore
;*
;******
wait_vb:
wv_lp1:
	bit	$d011
	bpl	wv_lp1
wv_lp2:
	bit	$d011
	bmi	wv_lp2
	rts

;**************************************************************************
;*
;* NAME  check_pal
;*
;* DESCRIPTION
;*   Check if pal.
;*   C=1 means pal
;*
;*   
;******
check_pal:
;Wait for the negative part of the screen
cp_lp1:
	lda	$d011
	bmi	cp_lp1
cp_lp2:
	lda	$d011
	bpl	cp_lp2
;check for pal
cp_lp3:
	lda	$d012
	cmp	#$20
	beq	cp_ex1		; C=1
	lda	$d011
	bmi	cp_lp3
	clc
cp_ex1:
	rts

;**************************************************************************
;*
;* SECTION  external code
;*   
;******
install_code:
	ldy	#>[FORMAT2_LEN+$ff]
	ldx	#0
ic_lp1:
ic_sm1:
	lda	format2_st,x
ic_sm2:
	sta	format2,x
	inx
	bne	ic_lp1
	inc	ic_sm1+2
	inc	ic_sm2+2
	dey
	bne	ic_lp1
	rts

format2_st:
	rorg	$c000
format2:
	incbin	"format2_c000.prg",2
FORMAT2_LEN	equ	.-format2
	rend
; eof
